package com.cgi.cihr;

import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import com.cgi.cihr.model.CheckOut;
import com.cgi.cihr.model.HRVolume;
import com.cgi.cihr.model.Pair;
import com.cgi.cihr.model.ProffBox;
import com.cgi.cihr.model.ProffDocument;
import com.cgi.cihr.model.ProffVolume;
import com.cgi.cihr.model.ResearchApplication;
import com.cgi.cihr.readers.BoxReader;
import com.cgi.cihr.readers.COReader;
import com.cgi.cihr.readers.ProffReader;
import com.cgi.cihr.writers.ControlFileWriter;
import com.cgi.cihr.writers.TextControlFileWriter;
import com.cgi.cihr.writers.XmlControlFileWriter;

/**
 * Migrate the PROFF data into Livelink.
 * 
 * @author tomas.hansson
 */
public class Migrate {
	private static final int MAX_FOLDERS = 6;
	
	private static Properties props;
	
	/**
	 * Main class.
	 * 
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		props = new Properties();
		props.load(ClassLoader.getSystemResourceAsStream("proff-stage.properties"));
		
		boolean debug = (props.getProperty("debug").equals("true"));
		
		ControlFileWriter w = null;
			
		if (debug) {
			w = new TextControlFileWriter(props.getProperty("controlFolder"));
		} else {
			w = new XmlControlFileWriter(props.getProperty("controlFolder"));			
		}
		
		String csvFolder = props.getProperty("csvFolder");

		ProffReader pr = new ProffReader(new FileReader(csvFolder + "PROFF.csv"));
		COReader cr = new COReader(new FileReader(csvFolder + "R_CO.csv"));
		BoxReader br = new BoxReader(new FileReader(csvFolder + "R_BOX.csv"));

		// Checked-out items and Boxes
		Map<Integer, Map<String, ? super Object>> coMap = cr.readAll();		
		Map<String, Map<String, ? super Object>> boxMap = br.readAll();
		
		// Write the hierarchy
		Map<String, List<Integer>> boxListMap = writeHierarchy(w, pr, coMap);
		
		// Write the boxes
		Writer ew = new FileWriter(props.getProperty("errorFile"));
		writeBoxes(w, ew, boxMap, boxListMap);
				
		pr.close();
		cr.close();
		br.close();
		ew.close();
		w.closeWriter();
	}
		
	private static Map<String, List<Integer>> writeHierarchy(ControlFileWriter w, ProffReader pr,
			Map<Integer, Map<String, ? super Object>> coMap) throws IOException {
		Map<String, ? super Object> prMap = pr.read();

		int level = 0;
		String root = null;
		Pair prevVolume = new Pair();
		Pair[] folders = new Pair[MAX_FOLDERS];
		String[] parents = new String[MAX_FOLDERS];
		Map<String, List<Integer>> boxListMap = new HashMap<String, List<Integer>>();
		
		Writer cw = new BufferedWriter(new FileWriter(props.getProperty("codesFile")));
		
		while (prMap != null) {
			// ROOT - just write the folder
			if (prMap.get("objectType").equals("ROOT")) {
				root = protect(prMap.get("title"));
				String desc = prMap.get("description") == null ? "" : (String) prMap.get("description");
				w.createFolder(props.getProperty("rootLocation"), root, desc, null);
			
			// FOLDER - store the folder hierarchy and use it to recreate the desired path
			} else if (((String) prMap.get("objectType")).startsWith("FOLDER")) {
				level = Integer.parseInt(((String) prMap.get("objectType")).substring(6));
				
				// Clear folders below
				for (int i = level; i < MAX_FOLDERS; i++) {
					folders[i] = null;
				}
				
				String title = protect(prMap.get("title"));
				String seriesClass = (String) prMap.get("seriesClass");
				String desc = prMap.get("description") == null ? "" : (String) prMap.get("description");
				parents = seriesClass.split("-", MAX_FOLDERS);
				
				for (int i = 1; i < parents.length; i++) {
					parents[i] = parents[i - 1] + "-" + parents[i];
				}
				
				folders[level] = new Pair(title, seriesClass);
				StringBuffer location = new StringBuffer(props.getProperty("rootLocation") + ":" + root);
				
				for (int i = 0; i < level; i++) {
					if (folders[i] != null && folders[i].getSecond().equals(parents[i])) {
						location.append(":" + folders[i].getFirst());
					}
				}
				
				w.createFolder(location.toString(), title, desc, null);				
			
			// VOLUME - Gather category info. Determine type and link to folder based on series class
			} else if (prMap.get("objectType").equals("VOLUME")) {
				int barCode = (Integer) prMap.get("barCode");
				String title = protect(prMap.get("title"));
				String seriesClass = (String) prMap.get("seriesClass");
				String description = prMap.get("description") == null ? "" : (String) prMap.get("description");

				ProffVolume pv = new ProffVolume(barCode, (Integer) prMap.get("volumeNo"),
						(String) prMap.get("folderStatus"), (String) prMap.get("folderStartDate"), 
						(String) prMap.get("folderEndDate"), (String) prMap.get("folderLocation"), 
						(String) prMap.get("folderBox"), (String) prMap.get("folderDispDate"),
						(String) prMap.get("seriesClass"), (String) prMap.get("author"), 
						(String) prMap.get("createdDate"));
				
				int objId = 0;
				ResearchApplication ra = null;
				HRVolume hr = null;
				String securityLevel = null;
				
				if (prMap.get("rootClass").equals(props.get("HRRoot"))) {
					objId = Integer.parseInt(props.getProperty("volumeObjId"));
					hr = new HRVolume((String) prMap.get("pin"));
				} else if (prMap.get("pin") != null) {
					objId = Integer.parseInt(props.getProperty("raObjId"));
					ra = new ResearchApplication((String) prMap.get("competition"), (String) prMap.get("appNumber"),
							(String) prMap.get("pin"), (String) prMap.get("frn"), description, title);
					securityLevel = props.getProperty("securityLevelB");
				} else {
					objId = Integer.parseInt(props.getProperty("volumeObjId"));
				}
				
				CheckOut co = null;
				
				// Checked-out?
				if (coMap.containsKey(barCode)) {
					cw.write(barCode + "\n");

					Map<String, ? super Object> coInfo = coMap.get(barCode);					
					co = new CheckOut(props.getProperty("coUser"), ((String) coInfo.get("userId")).toLowerCase(),
							props.getProperty("physicalLocation"), (Date) coInfo.get("coSubmittedDate"), 
							(Date) coInfo.get("recallDate"));
				}
				
				// Box?
				String box = (String) prMap.get("folderBox");
				
				if (box != null) {
					if (boxListMap.containsKey(box)) {
						boxListMap.get(box).add(barCode);
					} else {
						List<Integer> bcList = new ArrayList<Integer>();
						bcList.add(barCode);
						boxListMap.put(box, bcList);
					}
				}
							
				// Determine the parent
				StringBuffer location = new StringBuffer(props.getProperty("rootLocation") + ":" + root);
				int parentLevel = determineParent(folders, level, seriesClass);
				
				for (int i = 0; i <= parentLevel; i++) {
					if (folders[i] != null && folders[i].getSecond().equals(parents[i])) {
						location.append(":" + folders[i].getFirst());
					}
				}
				
				prevVolume = new Pair(seriesClass, location.toString() + ":" + title);
				
				if (title.length() > 250) {
					title = trunc(title);
				}

				w.createVolume((Integer) prMap.get("barCode"), location.toString(), title.replace(':', '-'), 
						description, objId, props.getProperty("physicalLocation"), 
						securityLevel, co, ra, pv, hr);
			
			// DOCUMENT - Gather category info. Determine type and link to folder based on series class
			} else if (prMap.get("objectType").equals("DOCUMENT")) {
				String title = protect(prMap.get("title")).replace(':', '-');
				int docNo = (Integer) prMap.get("documentNo");
				String seriesClass = (String) prMap.get("seriesClass");
				String description = prMap.get("description") == null ? "" : (String) prMap.get("description");

				ProffDocument pd = new ProffDocument(docNo, seriesClass, (String) prMap.get("documentType"), 
						(String) prMap.get("receivedDate"), (String) prMap.get("createdDate"), 
						(String) prMap.get("toPerson"), (String) prMap.get("fromPerson"), 
						(String) prMap.get("documentDispDate"), prMap.get("masterDocument"));
												
				// Determine the parent
				StringBuffer location = null;
				
				if (prevVolume.getFirst().equals(seriesClass)) {
					location = new StringBuffer((String) prevVolume.getSecond());
				} else {
					int parentLevel = determineParent(folders, level, seriesClass);
					location = new StringBuffer(props.getProperty("rootLocation") + ":" + root);
					
					for (int i = 0; i <= parentLevel; i++) {
						if (folders[i] != null && folders[i].getSecond().equals(parents[i])) {
							location.append(":" + folders[i].getFirst());
						}
					}
				}
				
				if (title.length() > 250) {
					title = trunc(title);
				}

				w.createDocument(docNo, location.toString(), title, description, 
					props.getProperty("documentObjId"), props.getProperty("physicalLocation"), null, pd);
			}
			
			prMap = pr.read();
		}			
		
		cw.close();
		
		return boxListMap;
	}
		
	private static void writeBoxes(ControlFileWriter w, Writer errWriter,
			Map<String, Map<String, ? super Object>> boxMap,
			Map<String, List<Integer>> boxListMap) throws IOException {		
		for (String boxNo : boxListMap.keySet()) {			
			if (boxMap.containsKey(boxNo)) {
				Map<String, ? super Object> box = boxMap.get(boxNo);
				
				ProffBox pb = new ProffBox((String) box.get("boxTitle"), (String) box.get("boxLocation"),
						(String) box.get("accessNo"), (Date) box.get("boxDisposalDate"));
				
				String location = props.getProperty("rootLocation") + ":" + props.getProperty("boxesFolder") + 
					":" + boxNo.substring(0, 4);
				w.createBox(location, boxNo, 
						props.getProperty("boxObjId"), props.getProperty("physicalLocation"),
						null, pb, boxListMap.get(boxNo));				
			} else {
				errWriter.write("Referenced box " + boxNo + " doesn't exist.\n");
			}
		}
		
		// Reference check
		for (String boxNo : boxMap.keySet()) {
			if (!boxListMap.containsKey(boxNo)) {
				errWriter.write("Box " + boxNo + " is never referenced.\n");				
			}
		}
				
		w.closeWriter();
	}
	
	private static int determineParent(Pair[] folders, int level, String seriesClass) {
		int parentLevel = -1;
		boolean found = false;
		String sc = seriesClass;
		
		while (parentLevel < 0) {
			for (int i = level; i >= 0; i--) {
				if (folders[i] != null && folders[i].getSecond().equals(sc)) {
					found = true;
					parentLevel = i;
					break;
				}
			}
			
			if (sc.lastIndexOf('-') >= 0 && !found) {
				sc = sc.substring(0, sc.lastIndexOf('-'));
			} else {
				break;
			}
		}
		
		return parentLevel;
	}
	
	private static String protect(Object o) {
		String newTxt = null;
		
		if (o == null) {
			return null;
		}
		
		if (o instanceof String) {
			String txt = (String) o;
			newTxt = txt.trim().replace(':', '-');
		}
		
		return newTxt;
	}
	
	private static String trunc(String txt) {
		return txt.substring(0, 120) + "..." + txt.substring(txt.length() - 120);
	}
}
