package com.cgi.cihr.readers;

import java.io.IOException;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;

import org.supercsv.cellprocessor.Optional;
import org.supercsv.cellprocessor.ParseDate;
import org.supercsv.cellprocessor.ParseInt;
import org.supercsv.cellprocessor.constraint.StrNotNullOrEmpty;
import org.supercsv.cellprocessor.constraint.Strlen;
import org.supercsv.cellprocessor.constraint.Unique;
import org.supercsv.cellprocessor.ift.CellProcessor;
import org.supercsv.io.CsvMapReader;
import org.supercsv.prefs.CsvPreference;

/**
 * CSV Box reader.
 * 
 * @author tomas.hansson
 */
public class BoxReader {
	private CsvMapReader csvReader;
	
	private CellProcessor[] processors = new CellProcessor[] {
			new Unique(),		 						// BOX_NO
			new Optional(),								// BOX_DESCRIPTION
			new Strlen(3),								// BOX_TYPE
			new Optional(),								// BOX_LOCATION
			new ParseDate("yyyy/MM/dd"),				// BOX_STARTDATE
			new ParseDate("yyyy/MM/dd"),				// BOX_ENDDATE
			new ParseInt(),								// BOX_PERIOD
			new ParseDate("yyyy/MM/dd"),				// BOX_DISPOSAL_DATE
			new Strlen(2),								// BOX_DISMTH
			new StrNotNullOrEmpty(),					// ROOT_CLASS
			new Optional(),								// COMPANY_CODE
			new Optional(),								// RECORD_ID
			new Optional(new ParseDate("yyyy/MM/dd")),	// DATE_CREATED
			new Optional(new ParseDate("yyyy/MM/dd")),	// BOX_REVIEW_DATE
			new Strlen(1),								// CONTENT_IND
			new ParseInt(),								// TRANSMITTAL_NO
			new Optional(),								// BOX_STATUS
			new Optional(),								// ACCESS_NO
			new Optional(),								// ACESS_LOCATION
			new Optional(),								// REFERENCE_NO
			new Optional()								// BOX_TITLE
		};
		
	private String[] header = new String[] {
			"boxNo", "boxDescription", "boxType", "boxLocation", "boxStartDate", "boxEndDate",
			"boxPeriod", "boxDisposalDate", "boxDismth", "rootClass", "companyCode", "recordId", "dateCreated",
			"boxReviewDate", "contentInd", "transmittalNo", "boxStatus", "accessNo", "accessLocation",
			"refernceNo", "boxTitle"
		};

	/**
	 * Constructor.
	 * 
	 * @param reader
	 */
	public BoxReader(Reader reader) {
		csvReader = new CsvMapReader(reader, CsvPreference.STANDARD_PREFERENCE);
		
		// Skip the header row
		try {
			csvReader.getCSVHeader(false);
		} catch (IOException e) {
			// Do nothing
		}
	}
	
	/**
	 * Read the map from CVS file.
	 * 
	 * @return Map<String, ? super Object>
	 * @throws IOException
	 */
	public Map<String, ? super Object> read() throws IOException {
		Map<String, ? super Object> m = csvReader.read(header, processors);
		
		return m;
	}
	
	/**
	 * Read all entries.
	 * 
	 * @return Map<String, Map<String, ? super Object>>
	 * @throws IOException
	 */
	public Map<String, Map<String, ? super Object>> readAll() throws IOException {
		Map<String, Map<String, ? super Object>> folderMap = new HashMap<String, Map<String,? super Object>>();
		
		Map<String, ? super Object> m = read();
		
		while (m != null) {
			folderMap.put((String) m.get("boxNo"), m);
			m = read();
		}
		
		csvReader.close();
		
		return folderMap;
	}

	/**
	 * Close.
	 * 
	 * @throws IOException
	 */
	public void close() throws IOException {
		csvReader.close();
	}
}
