package com.cgi.cihr.readers;

import java.io.IOException;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;

import org.supercsv.cellprocessor.Optional;
import org.supercsv.cellprocessor.ParseDate;
import org.supercsv.cellprocessor.ParseInt;
import org.supercsv.cellprocessor.constraint.StrNotNullOrEmpty;
import org.supercsv.cellprocessor.constraint.Strlen;
import org.supercsv.cellprocessor.constraint.Unique;
import org.supercsv.cellprocessor.ift.CellProcessor;
import org.supercsv.io.CsvMapReader;
import org.supercsv.prefs.CsvPreference;

/**
 * CSV Checked-Out reader.
 * 
 * @author tomas.hansson
 */
public class COReader {
	private CsvMapReader csvReader;
	
	private CellProcessor[] processors = new CellProcessor[] {
			new Unique(new ParseInt()),		 			// BAR_CODE
			new StrNotNullOrEmpty(),					// BAR_CODE_TYPE
			new StrNotNullOrEmpty(),					// USER_ID
			new Optional(new ParseInt()),				// ORDER_NO
			new ParseInt(),								// CO_PERIOD
			new StrNotNullOrEmpty(),					// FOLDER_TITLE
			new Optional(),								// FOLDER_LOCATION
			new ParseDate("yyyy/MM/dd"),				// CO_SUBMITTED_DATE
			new ParseDate("yyyy/MM/dd h:mm:ss a"),			// CO_SUBMITTED_TIME
			new ParseDate("yyyy/MM/dd"),				// CO_WAND_DATE
			new ParseDate("yyyy/MM/dd h:mm:ss a"),			// CO_WAND_TIME
			new ParseDate("yyyy/MM/dd"),				// RECALL_DATE
			new ParseInt(),								// FOLDER_NO
			new StrNotNullOrEmpty(new Strlen(1)),		// FOLDER_DISP_STATUS
			new Optional(new ParseInt())				// BOX_NO
		};
		
	private String[] header = new String[] {
			"barCode", "barCodeType", "userId", "orderNo", "coPeriod", "folderTitle", "folderLocation",
			"coSubmittedDate", "coSubmittedTime", "coWantedDate", "coWantedTime", "recallDate", 
			"folderNo", "folderDispStatus", "boxNo"
		};

	/**
	 * Constructor.
	 * 
	 * @param reader
	 */
	public COReader(Reader reader) {
		csvReader = new CsvMapReader(reader, CsvPreference.STANDARD_PREFERENCE);
		
		// Skip the header row
		try {
			csvReader.getCSVHeader(false);
		} catch (IOException e) {
			// Do nothing
		}
	}
	
	/**
	 * Read the map from CVS file.
	 * 
	 * @return Map<String, ? super Object>
	 * @throws IOException
	 */
	public Map<String, ? super Object> read() throws IOException {
		Map<String, ? super Object> m = csvReader.read(header, processors);
		
		return m;
	}
	
	/**
	 * Read all entries.
	 * 
	 * @return Map<Integer, Map<String, ? super Object>>
	 * @throws IOException
	 */
	public Map<Integer, Map<String, ? super Object>> readAll() throws IOException {
		Map<Integer, Map<String, ? super Object>> folderMap = new HashMap<Integer, Map<String,? super Object>>();
		
		Map<String, ? super Object> m = read();
		
		while (m != null) {
			folderMap.put((Integer) m.get("barCode"), m);
			m = read();
		}
		
		csvReader.close();
		
		return folderMap;
	}

	/**
	 * Close.
	 * 
	 * @throws IOException
	 */
	public void close() throws IOException {
		csvReader.close();
	}
}
