package com.cgi.cihr.writers;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.List;

import com.cgi.cihr.model.CheckOut;
import com.cgi.cihr.model.HRVolume;
import com.cgi.cihr.model.ProffBox;
import com.cgi.cihr.model.ProffDocument;
import com.cgi.cihr.model.ProffVolume;
import com.cgi.cihr.model.ResearchApplication;

/**
 * Control filer writer.
 * 
 * @author tomas.hansson
 */
public abstract class ControlFileWriter {
	private static final int MAX_RECORDS = 5000;
	private static final String FILENAME = "control";
	
	private int recCount;
	private Writer writer;
	private String recsLocation;
	private String suffix;
	private boolean isXml;
	
	protected ControlFileWriter(String location, String suffix, boolean isXml) throws IOException {
		this.recsLocation = location;
		this.suffix = suffix;
		this.isXml = isXml;
		
		File loc = new File(recsLocation);
		
		if (loc.exists() && !loc.isDirectory()) {
			throw new IOException();
		}
		
		if (!loc.exists()) {
			loc.mkdirs();
		}
	}
	
	/**
	 * Open writer.
	 * 
	 * @throws IOException
	 */
	public void openWriter() throws IOException {
		recCount = 1;
		
		int fileCount = 1;
		File file = new File(recsLocation, FILENAME + "-" + String.format("%05d", fileCount) + "." + suffix);
		
		while (file.exists()) {
			file = new File(recsLocation, FILENAME + "-" + String.format("%05d", ++fileCount) + "." + suffix);
		}
		
		writer =  new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8")); 
		
		if (isXml) {
			writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			writer.write("<import>\n");
		}
	}
	
	/**
	 * Increment writer.
	 * 
	 * @throws IOException
	 */
	protected void incrementWriter()  throws IOException {
		closeWriter();
		openWriter();
	}
	
	/**
	 * Write a record.
	 * 
	 * @param record
	 * @throws IOException
	 */
	public void writeRecord(String record) throws IOException {
		if (writer == null) {
			openWriter();
		}
		
		writer.write(record);
		
		if (++recCount > MAX_RECORDS) {
			incrementWriter();
		}
	}
	
	/**
	 * Text info for folder.
	 * 
	 * @param location
	 * @param title
	 * @param description
	 * @param securityLevel
	 * @throws IOException
	 */
	public abstract void createFolder(String location, String title, 
			String description, String securityLevel) throws IOException;
	
	/**
	 * Text info for volume.
	 * 
	 * @param id
	 * @param location
	 * @param title
	 * @param description
	 * @param objId
	 * @param physicalLocation
	 * @param securityLevel
	 * @param co
	 * @param ra
	 * @param pv
	 * @param hr
	 * @throws IOException
	 */
	public abstract void createVolume(int id, String location, String title, String description,
			int objId, String physicalLocation, String securityLevel,
			CheckOut co, ResearchApplication ra, ProffVolume pv, HRVolume hr) throws IOException;

	/**
	 * XML to create a physical document.
	 * 
	 * @param docNo
	 * @param location
	 * @param title
	 * @param description
	 * @param documentObjId
	 * @param physicalLocation
	 * @param securityLevel
	 * @param pd
	 * @throws IOException
	 */
	public abstract void createDocument(int docNo, String location, String title, String description, 
			String documentObjId, String physicalLocation, String securityLevel,			
			ProffDocument pd) throws IOException;

	/**
	 * Create a box with included volumes.
	 * 
	 * @param location
	 * @param title
	 * @param boxObjId
	 * @param physicalLocation
	 * @param securityLevel
	 * @param pb
	 * @param idList
	 * @throws IOException
	 */
	public abstract void createBox(String location, String title, 
			String boxObjId, String physicalLocation, String securityLevel,
			ProffBox pb, List<Integer> idList) throws IOException;

	/**
	 * Close the writer.
	 * 
	 * @throws IOException
	 */
	public void closeWriter() throws IOException {
		if (writer != null) {
			if (isXml) {
				writer.write("</import>\n");
			}
			
			writer.close();
			writer = null;
		}
	}
	
	/**
	 * Get record count.
	 * 
	 * @return int
	 */
	public int getRecCount() {
		return recCount;
	}
}
