package com.cgi.cihr.writers;

import java.io.IOException;
import java.util.List;

import org.apache.commons.lang.StringEscapeUtils;

import com.cgi.cihr.model.CheckOut;
import com.cgi.cihr.model.HRVolume;
import com.cgi.cihr.model.ProffBox;
import com.cgi.cihr.model.ProffDocument;
import com.cgi.cihr.model.ProffVolume;
import com.cgi.cihr.model.ResearchApplication;


/**
 * Writer for control XML files.
 * 
 * @author tomas.hansson
  */
public class XmlControlFileWriter extends ControlFileWriter {
	private static final String SUFFIX = "xml";
	
	/**
	 * Constructor.
	 * 
	 * @param location
	 * @throws IOException
	 */
	public XmlControlFileWriter(String location) throws IOException {
		super(location, SUFFIX, true);
	}

	/**
	 * XML to create a folder.
	 * 
	 * @param location
	 * @param title
	 * @param description
	 * @param securityLevel
	 * @throws IOException
	 */
	public void createFolder(String location, String title, 
			String description, String securityLevel) throws IOException {
		StringBuffer s = new StringBuffer();
		
		s.append("  <node type=\"folder\" action=\"sync\">\n");
		s.append("    <location>" + StringEscapeUtils.escapeXml(location) + "</location>\n");
		s.append("    <title language=\"en_CA\">" + StringEscapeUtils.escapeXml(title) + "</title>\n");
		s.append("    <description language=\"en_CA\">" + StringEscapeUtils.escapeXml(description) + "</description>\n");
		
		if (securityLevel != null) {
			s.append("    <securityclearance>\n");
			s.append("      <securitylevel>" + securityLevel + "</securitylevel>\n");
			s.append("    </securityclearance>\n");
		}
		
		s.append("  </node>\n");
		
		writeRecord(s.toString());
	}
	
	/**
	 * XML to create a volume (physical folder).
	 * 
	 * @param id
	 * @param location
	 * @param title
	 * @param description
	 * @param objId
	 * @param physicalLocation
	 * @param securityLevel
	 * @param co
	 * @param ra
	 * @param pv
	 * @param hr
	 * @throws IOException
	 */
	public void createVolume(int id, String location, String title, String description,
			int objId, String physicalLocation, String securityLevel,
			CheckOut co, ResearchApplication ra, ProffVolume pv, HRVolume hr) throws IOException {
		StringBuffer s = new StringBuffer();
		
		s.append("  <node type=\"physicalitem_container\" action=\"create\">\n");
		s.append("    <location>" + StringEscapeUtils.escapeXml(location) + "</location>\n");
		s.append("    <title language=\"en_CA\">" + StringEscapeUtils.escapeXml(title) + "</title>\n");
		s.append("    <description language=\"en_CA\">" + StringEscapeUtils.escapeXml(description) + "</description>\n");
		s.append("    <mediatype>" + objId + "</mediatype>\n");
		s.append("    <physicallocation>" + physicalLocation + "</physicallocation>\n");
		s.append("    <uniqueID>" + id + "</uniqueID>\n");
		
		if (securityLevel != null) {		
			s.append("    <securityclearance>\n");
			s.append("      <securitylevel>" + securityLevel + "</securitylevel>\n");
			s.append("    </securityclearance>\n");
		}
		
		if (co != null) {
			s.append(co.toXMLString());
		}
		
		if (ra != null) {
			s.append(ra.toXMLString());
		}
		
		if (pv != null) {
			s.append(pv.toXMLString());
		}
		
		if (hr != null) {
			s.append(hr.toXMLString());
		}
		
		s.append("  </node>\n");
		
		writeRecord(s.toString());		
	}
	
	/**
	 * XML to create a physical document.
	 * 
	 * @param docNo
	 * @param location
	 * @param title
	 * @param description
	 * @param documentObjId
	 * @param physicalLocation
	 * @param securityLevel
	 * @param pd
	 * @throws IOException
	 */
	public void createDocument(int docNo, String location, String title, String description, 
			String documentObjId, String physicalLocation, String securityLevel,
			ProffDocument pd) throws IOException {
		StringBuffer s = new StringBuffer();
		
		s.append("  <node type=\"physicalitem\" action=\"create\">\n");
		s.append("    <location>" + StringEscapeUtils.escapeXml(location) + "</location>\n");
		s.append("    <title language=\"en_CA\">" + StringEscapeUtils.escapeXml(title) + "</title>\n");
		s.append("    <description language=\"en_CA\">" + StringEscapeUtils.escapeXml(description) + "</description>\n");
		s.append("    <mediatype>" + documentObjId + "</mediatype>\n");
		s.append("    <physicallocation>" + physicalLocation + "</physicallocation>\n");
		s.append("    <uniqueID>D" + docNo + "</uniqueID>\n");
		
		if (securityLevel != null) {
			s.append("    <securityclearance>\n");
			s.append("      <securitylevel>" + securityLevel + "</securitylevel>\n");
			s.append("    </securityclearance>\n");
		}
		
		if (pd != null) {
			s.append(pd.toXMLString());
		}
		
		s.append("  </node>\n");
		
		writeRecord(s.toString());		
	}	
	
	/**
	 * Create a box with included volumes.
	 * 
	 * @param location
	 * @param title
	 * @param boxObjId
	 * @param physicalLocation
	 * @param securityLevel
	 * @param pb
	 * @param idList
	 * @throws IOException
	 */
	public void createBox(String location, String title, 
			String boxObjId, String physicalLocation, String securityLevel,
			ProffBox pb, List<Integer> idList) throws IOException {
		StringBuffer s = new StringBuffer();
		
		s.append("  <node type=\"physicalitem_box\" action=\"create\">\n");
		s.append("    <location>" + StringEscapeUtils.escapeXml(location) + "</location>\n");
		s.append("    <title>" + StringEscapeUtils.escapeXml(title) + "</title>\n");
		s.append("    <mediatype>" + boxObjId + "</mediatype>\n");
		s.append("    <physicallocation>" + physicalLocation + "</physicallocation>\n");
		s.append("    <uniqueID>" + StringEscapeUtils.escapeXml(title) + "</uniqueID>\n");

		if (securityLevel != null) {
			s.append("    <securityclearance>\n");
			s.append("      <securitylevel>" + securityLevel + "</securitylevel>\n");
			s.append("    </securityclearance>\n");
		}
		
		s.append("    <assign type=\"unique\">\n");
		
		for (int id : idList) {
			s.append("      <id>" + id + "</id>\n");
		}
		
		s.append("    </assign>\n");
		
		if (pb != null) {
			s.append(pb.toXMLString());
		}
		
		s.append("  </node>\n");
		
		writeRecord(s.toString());		
	}			
}
